"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const identity_1 = tslib_1.__importDefault(require("lodash/identity"));
const listr_1 = tslib_1.__importDefault(require("listr"));
const yn_1 = tslib_1.__importDefault(require("yn"));
const utils = tslib_1.__importStar(require("./utils"));
const prompts_1 = require("./prompts");
const constants_1 = require("./constants");
const AZS_COUNT = 3;
exports.configureKYCServicesStack = (conf, { truefaceSpoof, rankOne, client, mycloudStackName }) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const servicesStackName = utils.getServicesStackName(mycloudStackName);
    const servicesStackId = yield utils.getStackId(client, servicesStackName);
    const exists = !!servicesStackId;
    const bucket = yield conf.getPrivateConfBucket();
    const discoveryObjPath = `${bucket}/discovery/ecs-services.json`;
    const tfVerb = truefaceSpoof ? 'enable' : 'disable';
    const roVerb = rankOne ? 'enable' : 'disable';
    yield prompts_1.confirmOrAbort(`${tfVerb} TrueFace Spoof, ${roVerb} RankOne?`);
    const repoNames = [
        constants_1.REPO_NAMES.nginx,
        truefaceSpoof && constants_1.REPO_NAMES.truefaceSpoof,
        rankOne && constants_1.REPO_NAMES.rankOne,
    ].filter(identity_1.default).join(', ');
    yield prompts_1.confirmOrAbort(`has Tradle given you access to the following ECR repositories? ${repoNames}`);
    const azsCount = AZS_COUNT;
    const { region, availabilityZones } = exists
        ? yield getServicesStackInfo(client, { stackId: servicesStackId })
        : yield chooseRegionAndAZs(client, { count: azsCount });
    const enableSSH = yn_1.default(yield prompts_1.confirm('enable SSH into the instances?', false));
    const parameters = availabilityZones.map((az, i) => ({
        ParameterKey: `AZ${(i + 1)}`,
        ParameterValue: az
    }));
    parameters.push({
        ParameterKey: 'S3PathToWriteDiscovery',
        ParameterValue: discoveryObjPath
    });
    if (truefaceSpoof) {
        parameters.push({
            ParameterKey: 'EnableTruefaceSpoof',
            ParameterValue: 'true'
        });
    }
    if (rankOne) {
        parameters.push({
            ParameterKey: 'EnableRankOne',
            ParameterValue: 'true',
        });
    }
    if (enableSSH) {
        const key = yield prompts_1.chooseEC2KeyPair(client);
        parameters.push({
            ParameterKey: 'KeyName',
            ParameterValue: key
        });
    }
    yield prompts_1.confirmOrAbort(`are you freaking ready?`);
    const tasks = [
        {
            title: 'validate template',
            task: (ctx) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                const params = {
                    StackName: servicesStackId || servicesStackName,
                    Parameters: parameters,
                    TemplateURL: constants_1.SERVICES_STACK_TEMPLATE_URL,
                    Capabilities: ['CAPABILITY_NAMED_IAM']
                };
                let method;
                if (exists) {
                    method = 'updateStack';
                }
                else {
                    method = 'createStack';
                    params.DisableRollback = true;
                }
                ctx.wait = yield utils[method](client, params);
            }),
        },
        {
            title: `create/update KYC services stack`,
            task: ctx => ctx.wait(),
        }
    ];
    yield new listr_1.default(tasks).run();
});
const getServicesStackInfo = (client, { stackId }) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const { Stacks } = yield client.cloudformation.describeStacks({
        StackName: stackId,
    }).promise();
    const { Outputs } = Stacks[0];
    const availabilityZones = getOutput(Outputs, 'AvailabilityZones').split(',');
    const region = getOutput(Outputs, 'Region');
    return {
        region,
        availabilityZones,
    };
});
const getOutput = (Outputs, key) => Outputs.find(o => o.OutputKey === key).OutputValue;
const chooseRegionAndAZs = (client, { count }) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const usedEIPCount = yield utils.getUsedEIPCount(client);
    if (usedEIPCount > 2) {
        yield prompts_1.confirmOrAbort(`your account has ${usedEIPCount} elastic ips in use. This stack will create ${count} more. Keep in mind that AWS's base limit is 5 per account. You can easily get them to relax that limit, but if you haven't yet, there's a chance this stack will fail.`);
    }
    const region = yield prompts_1.chooseRegion();
    const availabilityZones = yield prompts_1.chooseAZs(client, { region, count });
    return {
        region,
        availabilityZones,
    };
});
//# sourceMappingURL=kyc-services.js.map